@@ -4,11 +4,12 @@
 Standard forms
 """
 
-from flask import render_template, request, Markup, abort, flash, redirect, json, escape, url_for
+from pytz import utc, timezone as pytz_timezone
+import bleach
+from flask import render_template, request, Markup, abort, flash, redirect, json, escape, url_for, make_response
 from wtforms.widgets import html_params
 import flask.ext.wtf as wtf
 from coaster import make_name
-import bleach
 
 
 class RichText(wtf.widgets.TextArea):
@@ -56,7 +57,7 @@ class DateTimeInput(wtf.widgets.Input):
         if not value:
             value = ' '
         date_value, time_value = value.split(' ', 1)
-        return Markup(u'<input type="text" class="datetime-date" data-datepicker="datepicker" %s /> <input type="time" class="datetime-time" %s />' % (
+        return Markup(u'<input type="text" class="datetime-date" data-datepicker="datepicker" %s /> <input type="text" class="datetime-time" %s />' % (
             html_params(name=field.name, id=field_id + '-date', value=date_value, **kwargs),
             html_params(name=field.name, id=field_id + '-time', value=time_value, **kwargs)
             ))
@@ -84,33 +85,44 @@ class RichTextField(wtf.fields.TextAreaField):
 
             # Additional fields
             content_css=None,
-            buttons1=None, buttons2=None, buttons3=None,
-            blockformats=None,
-            width=None, height=None,
             linkify=True, nofollow=True,
-            valid_elements=None, sanitize_tags=None, sanitize_attributes=None, **kwargs):
+            tinymce_options=None,
+            sanitize_tags=None, sanitize_attributes=None, **kwargs):
 
         super(RichTextField, self).__init__(label=label, validators=validators, filters=filters,
             description=description, id=id, default=default, widget=widget, _form=_form, _name=_name,
             _prefix=_prefix, **kwargs)
 
-        if buttons1 is None:
-            buttons1 = "bold,italic,|,sup,sub,|,bullist,numlist,|,link,unlink,|,blockquote,|,removeformat,code"
-        if buttons2 is None:
-            buttons2 = ""
-        if buttons3 is None:
-            buttons3 = ""
-        if blockformats is None:
-            blockformats = "p,h3,h4,h5,h6,blockquote,dt,dd"
-        if width is None:
-            width = "100%"
-        if height is None:
-            height = "159"
-        # valid_elements and sanitize_tags/attributes are distinct because one is used by TinyMCE and
-        # the other by bleach. Their formats are incompatible and we're too lazy to write code to
-        # autogenerate one from the other.
-        if valid_elements is None:
-            valid_elements = "p,br,strong/b,em/i,sup,sub,h3,h4,h5,h6,ul,ol,li,a[!href|title|target],blockquote,code"
+        if tinymce_options is None:
+            tinymce_options = {}
+        else:
+            # Clone the dict to preserve local edits
+            tinymce_options = dict(tinymce_options)
+
+        # Set defaults for TinyMCE
+        tinymce_options.setdefault('theme', "advanced")
+        tinymce_options.setdefault('plugins', "")
+        tinymce_options.setdefault('theme_advanced_buttons1',
+            "bold,italic,|,sup,sub,|,bullist,numlist,|,link,unlink,|,blockquote,|,removeformat,code")
+        tinymce_options.setdefault('theme_advanced_buttons2', "")
+        tinymce_options.setdefault('theme_advanced_buttons3', "")
+        tinymce_options.setdefault('blockformats', "p,h3,h4,h5,h6,blockquote,dt,dd")
+        tinymce_options.setdefault('width', "100%")
+        tinymce_options.setdefault('height', "159")
+        tinymce_options.setdefault('valid_elements',
+            "p,br,strong/b,em/i,sup,sub,h3,h4,h5,h6,ul,ol,li,a[!href|title|target],blockquote,code")
+        tinymce_options.setdefault('theme_advanced_toolbar_location', "top")
+        tinymce_options.setdefault('theme_advanced_toolbar_align', "left")
+        tinymce_options.setdefault('theme_advanced_statusbar_location', "bottom")
+        tinymce_options.setdefault('theme_advanced_resizing', True)
+        tinymce_options.setdefault('theme_advanced_path', False)
+        tinymce_options.setdefault('relative_urls', False)
+
+        # Remove options that cannot be set by callers
+        tinymce_options.pop('content_css', None)
+        tinymce_options.pop('script_url', None)
+        tinymce_options.pop('setup', None)
+
         if sanitize_tags is None:
             sanitize_tags = ['p', 'br', 'strong', 'em', 'sup', 'sub', 'h3', 'h4', 'h5', 'h6',
                 'ul', 'ol', 'li', 'a', 'blockquote', 'code']
@@ -119,18 +131,15 @@ class RichTextField(wtf.fields.TextAreaField):
 
         self.linkify = linkify
         self.nofollow = nofollow
+        self.tinymce_options = tinymce_options
 
         self.content_css = content_css
-        self.buttons1 = buttons1
-        self.buttons2 = buttons2
-        self.buttons3 = buttons3
-        self.blockformats = blockformats
-        self.width = width
-        self.height = height
-        self.valid_elements = valid_elements
         self.sanitize_tags = sanitize_tags
         self.sanitize_attributes = sanitize_attributes
 
+    def tinymce_options_json(self):
+        return [(Markup(json.dumps(k)), Markup(json.dumps(v))) for k, v in self.tinymce_options.items()]
+
     def process_formdata(self, valuelist):
         super(RichTextField, self).process_formdata(valuelist)
         # Sanitize data
@@ -150,9 +159,61 @@ class DateTimeField(wtf.fields.DateTimeField):
     """
     widget = DateTimeInput()
 
-    def __init__(self, label=None, validators=None, format='%Y-%m-%d %I:%M %p', **kwargs):
+    def __init__(self, label=None, validators=None,
+            format='%Y-%m-%d %I:%M%p', timezone=None, **kwargs):
         super(DateTimeField, self).__init__(label, validators, **kwargs)
         self.format = format
+        self.timezone = timezone
+        if timezone:
+            self.tz = pytz_timezone(timezone)
+        else:
+            self.tz = utc
+
+    def _value(self):
+        if self.data:
+            if self.timezone:
+                if self.data.tzinfo is None:
+                    data = utc.localize(self.data).astimezone(self.tz)
+                else:
+                    data = self.data.astimezone(self.tz)
+            else:
+                data = self.data
+            value = data.strftime(self.format)
+        else:
+            value = ''
+        return value
+
+    def process_formdata(self, valuelist):
+        super(DateTimeField, self).process_formdata(valuelist)
+        if self.timezone:
+            # Convert from user timezone back to UTC, then discard tzinfo
+            self.data = self.tz.localize(self.data).astimezone(utc).replace(tzinfo=None)
+
+
+class HiddenMultiField(wtf.fields.TextField):
+    """
+    A hidden field that stores multiple comma-separated values, meant to be
+    used as an Ajax widget target. The optional ``separator`` parameter
+    can be used to specify an alternate separator character (default ``','``).
+    """
+    widget = wtf.HiddenInput()
+
+    def __init__(self, *args, **kwargs):
+        self.separator = kwargs.pop('separator', ',')
+        super(HiddenMultiField, self).__init__(*args, **kwargs)
+
+    def _value(self):
+        if self.data:
+            return self.separator.join(self.data)
+        else:
+            return ''
+
+    def process_formdata(self, valuelist):
+        super(HiddenMultiField, self).process_formdata(valuelist)
+        if not self.data:
+            self.data = []  # Calling ''.split(',') will give us [''] which is not "falsy"
+        else:
+            self.data = self.data.split(self.separator)
 
 
 class Form(wtf.Form):
@@ -220,38 +281,40 @@ def render_form(form, title, message='', formid='form', submit=u"Submit", cancel
         if isinstance(field.widget, wtf.FileInput):
             multipart = True
     if request.is_xhr and ajax:
-        return render_template('baseframe/ajaxform.html', form=form, title=title,
+        return make_response(render_template('baseframe/ajaxform.html', form=form, title=title,
             message=message, formid=formid, submit=submit,
-            cancel_url=cancel_url, multipart=multipart)
+            cancel_url=cancel_url, multipart=multipart))
     else:
-        return render_template('baseframe/autoform.html', form=form, title=title,
+        return make_response(render_template('baseframe/autoform.html', form=form, title=title,
             message=message, formid=formid, submit=submit,
-            cancel_url=cancel_url, ajax=ajax, multipart=multipart)
+            cancel_url=cancel_url, ajax=ajax, multipart=multipart))
 
 
-def render_message(title, message):
+def render_message(title, message, code=200):
     if request.is_xhr:
-        return Markup("<p>%s</p>" % escape(message))
+        return make_response(Markup("<p>%s</p>" % escape(message)), code)
     else:
-        return render_template('baseframe/message.html', title=title, message=message)
+        return make_response(render_template('baseframe/message.html', title=title, message=message), code)
 
 
 def render_redirect(url, code=302):
     if request.is_xhr:
-        return render_template('baseframe/redirect.html', quoted_url=Markup(json.dumps(url)))
+        return make_response(render_template('baseframe/redirect.html', quoted_url=Markup(json.dumps(url))))
     else:
         return redirect(url, code=code)
 
 
-def render_delete_sqla(ob, db, title, message, success=u'', next=None):
-    if not ob:
+def render_delete_sqla(obj, db, title, message, success=u'', next=None, cancel_url=None):
+    if not obj:
         abort(404)
     form = ConfirmDeleteForm()
-    if form.validate_on_submit():
-        if 'delete' in request.form:
-            db.session.delete(ob)
+    if request.method in ('POST', 'DELETE') and form.validate():
+        if 'delete' in request.form or request.method == 'DELETE':
+            db.session.delete(obj)
             db.session.commit()
             if success:
                 flash(success, "success")
-        return render_redirect(next or url_for('index'))
-    return render_template('baseframe/delete.html', form=form, title=title, message=message)
+            return render_redirect(next or url_for('index'))
+        else:
+            return render_redirect(cancel_url or next or url_for('index'))
+    return make_response(render_template('baseframe/delete.html', form=form, title=title, message=message))
