@@ -2,20 +2,24 @@
 // This is a global function. Isn't there a better way to do this?
 function activate_chosen(selector) {
   if (!navigator.userAgent.match(/(iPod|iPad|iPhone|Android)/)) {
-    $(selector).chosen({allow_single_deselect: true});
+    // $(selector).chosen({allow_single_deselect: true});
+    $(selector).select2({allowClear: true});
   }
 }
 
 $(function() {
   // Activate chosen on all 'select' tags.
-  activate_chosen('select');
+  activate_chosen('select:not(.notselect)');
 
   var matchtab = function() {
-    var url = document.location.toString();
+    var url = document.location.toString(), tabmatch = null;
     if (url.match('#/')) {
-      $('.nav-tabs.nav-tabs-auto a[href=#'+url.split('#/')[1]+']').tab('show');
+      tabmatch = $('.nav-tabs.nav-tabs-auto a[href="#'+url.split('#/')[1]+'"]');
     } else if (url.match('#')) {
-      $('.nav-tabs.nav-tabs-auto a[href=#'+url.split('#')[1]+']').tab('show');
+      tabmatch = $('.nav-tabs.nav-tabs-auto a[href="#'+url.split('#')[1]+'"]');
+    }
+    if (tabmatch !== null && tabmatch.length !== 0) {
+      $(tabmatch[0]).tab('show');
     }
   };
 
@@ -30,6 +34,9 @@ $(function() {
   var url = document.location.toString();
   if (!url.match('#')) {
     // Activate the first tab if none are active
-    $('.nav-tabs.nav-tabs-auto a').filter(':first').tab('show');
+    var tabmatch = $('.nav-tabs.nav-tabs-auto a').filter(':first');
+    if (tabmatch.length !== 0) {
+        $(tabmatch[0]).tab('show');
+    }
   }
 });
